Package: Rbec
Type: Package
Title: Rbec: a tool for analysis of amplicon sequencing data from
        synthetic microbial communities
Version: 1.20.0
Authors@R: person("Pengfan", "Zhang", email = "pzhang@mpipz.mpg.de", role = c("aut", "cre"))
Description: Rbec is a adapted version of DADA2 for analyzing amplicon sequencing data
    from synthetic communities (SynComs), where the reference sequences for each strain exists.
    Rbec can not only accurately profile the microbial compositions in SynComs, but also predict 
    the contaminants in SynCom samples.
License: LGPL-3
Imports: Rcpp (>= 1.0.6), dada2, ggplot2, readr, doParallel, foreach,
        grDevices, stats, utils
LinkingTo: Rcpp
RoxygenNote: 7.1.1
biocViews: Sequencing, MicrobialStrain, Microbiome
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/Rbec
git_branch: RELEASE_3_23
git_last_commit: 1d7e45a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:06:25 UTC; biocbuild
Author: Pengfan Zhang [aut, cre]
Maintainer: Pengfan Zhang <pzhang@mpipz.mpg.de>
