useDynLib(Rbec, .registration=TRUE)
importFrom(Rcpp, evalCpp)
importFrom("grDevices", "dev.off", "pdf")
importFrom("stats", "IQR", "dpois", "loess", "predict")
importFrom("utils", "write.table")
import(dada2)
import(readr)
import(ggplot2)
import(doParallel)
import(foreach)
export(Rbec)
export(Contam_detect)
