# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @name kmer_dist
#' @title DADA2
#' @description Calculate the kmer distance between two sequences. This function is originally developed by DADA2 in 2012 and we adapt it here.
#'
#' @param s1 A \code{character(1)} of DNA sequence 1.
#' @param s2 A \code{character(1)} of DNA sequence 2.
#' @param kmer_size Kmer size.
#' @return The kmer distance between two sequences
#' @noRd
#'
kmer_dist <- function(s1, s2, kmer_size) {
    .Call(`_Rbec_kmer_dist`, s1, s2, kmer_size)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call('_Rbec_RcppExport_registerCCallable', PACKAGE = 'Rbec')
})
